' PB File controls test - Public Domain by Borje Hagsten, January 2002.
' Shows how to use whole set of PB custom file controls.
'
' Note - all controls are totally self-redrawing via use of container
' control as real parent. Advantage is no need to pass on WM_DRAWITEM.
' Small disadvantage is the control cannot be accessed via DDT commands,
' one must use Windows API calls with the handle returned at creation.
'
' LOG:
' Jan 14, 2003: Changed handles to DWORD and now use GetSystemMetrics
'               in some controls to ensure item height is enough for
'               small icon, if big font is set. The controls now also
'               uses GetWindowLong(hWnd, %GWL_HINSTANCE) to get proper
'               instance handle at creation.
'
' Declares
'
#COMPILE EXE
#INCLUDE "WIN32API.INC"
#INCLUDE "PBDRIVE.INC"
#INCLUDE "PBDIR.INC"
#INCLUDE "PBFILE.INC"
#INCLUDE "CBFTYPE.INC"

%IDC_DRIVE1  = 3000
%IDC_DIR1    = 3010
%IDC_FILE1   = 3020
%IDC_FTYPE1  = 3030
%IDC_FILETXT = 3040
%IDC_PATHLBL = 3050

DECLARE CALLBACK FUNCTION DlgProc
DECLARE SUB CompactPathToLabel(BYVAL hDlg AS DWORD, BYVAL idLabel AS LONG, BYVAL path AS STRING)

'
' Entrance - Create dialog and controls
'
FUNCTION PBMAIN
  LOCAL hDlg AS DWORD, hDrive AS DWORD, hDir AS DWORD, hFile AS DWORD, hFtype AS DWORD

  DIALOG NEW 0, "PB file controls sample", , , 300, 150, _
             %WS_CAPTION OR %DS_MODALFRAME OR %WS_SYSMENU TO hDlg

  CONTROL ADD TEXTBOX, hDlg, %IDC_FILETXT,       "",          6,  11, 140,  12

  'Note: File control also handles '%LBS_EXTENDEDSEL or %LBS_MULTIPLESEL (4:th param from end)
  hFile  = CreatePBFileLB(hDlg, %IDC_FILE1,          6,  23, 140,  80, 0, 1, "", "*.TXT")
  hDir   = CreatePBDirLB(hDlg, %IDC_DIR1,          153,  23, 140,  80, 1, "")
  hFtype = CreatePBFileTypeCB(hDlg, %IDC_FTYPE1,     6, 109, 140, 120, 1)
  hDrive = CreatePBDriveCB(hDlg, %IDC_DRIVE1,      153, 109, 140, 120, 1, "")

  'initiate controls - best order is to begin with File type, then File, Dir and last Drive control.
  SendMessage hDir, %DCM_SETDIRSELTYPE, 0, 0       'wParam sets single-click action
  SendMessage hDir, %DCM_SETDIRFILECTL, hFile, 0   'wParam sets related file control (handle)
  SendMessage hDrive, %DCM_SETDRVDIRCTL, hDir, 0   'wParam sets related dir control (handle)

  CONTROL ADD CHECKBOX, hDlg, 3070, "Enabled/Disabled ",  5, 132,  80,  10
  CONTROL ADD CHECKBOX, hDlg, 3071, "3DFACE selcolor ",  90, 132,  80,  10
  CONTROL ADD BUTTON,   hDlg, 3060, "&Ok",              190, 131,  50,  14
  CONTROL ADD BUTTON,   hDlg, %IDCANCEL, "&Cancel",     244, 131,  50,  14
  CONTROL SET CHECK hDlg, 3070, 1

  CONTROL ADD LABEL, hDlg, 10,     "File:",         7,   1,  50,  10
  CONTROL ADD LABEL, hDlg, 11,     "File type:",    7,  99,  50,  10
  CONTROL ADD LABEL, hDlg, 12,     "Path:",       154,   1,  50,  10
  CONTROL ADD LABEL, hDlg, 13,     "Unit:",       154,  99,  50,  10
  CONTROL ADD LABEL, hDlg, %IDC_PATHLBL, "",      154,  12, 140,  10

 '------------------------------------------------------------------
 ' add some file types and associated icon data to file type control
 ' (which in reality is nothing but a plain ownerdrawn combobox that looks
 ' for and draws an icon on left side, if ITEMDATA has a handle for one..)
 '------------------------------------------------------------------
  LOCAL ii AS LONG, sfi AS SHFILEINFO, zTxt AS ASCIIZ * 200
  REDIM fTypes(6) AS STRING

  'Create some file types for list
  fTypes(0)  = "*.*"   : fTypes(1)  = "*.BAS"
  fTypes(2)  = "*.BAT" : fTypes(3)  = "*.DOC"
  fTypes(4)  = "*.RTF" : fTypes(5)  = "*.TXT"
  fTypes(6)  = "*.ZIP"
  FOR ii = 0 TO UBOUND(fTypes)
     zTxt = fTypes(ii)

     'get file type info and associated icon
     SHGetFileInfo zTxt, %FILE_ATTRIBUTE_NORMAL, sfi, LEN(sfi), _
                         %SHGFI_USEFILEATTRIBUTES OR %SHGFI_TYPENAME OR _
                         %SHGFI_SYSICONINDEX OR %SHGFI_ICON OR %SHGFI_SMALLICON

     IF ii > 0 THEN
        zTxt = sfi.szTypeName + "  (" + zTxt + ")"
     ELSE
        zTxt = "All files  (" + zTxt + ")"
     END IF

     SendMessage hFtype, %CB_ADDSTRING, 0, BYVAL VARPTR(zTxt)
     SendMessage hFtype, %CB_SETITEMDATA, ii, sfi.hIcon
  NEXT
  SendMessage hFtype, %CB_SETCURSEL, 5, 0  'pre-select *.TXT
'-----------------------------------------------
  DIALOG DOEVENTS

  DIALOG SHOW MODAL hDlg, CALL DlgProc

END FUNCTION

'
' Main dialog callback
'
CALLBACK FUNCTION DlgProc
  SELECT CASE CBMSG
     CASE %WM_INITDIALOG
        LOCAL ii AS LONG, lRes AS LONG, cSel AS LONG, _
              hDrive AS DWORD, hDir AS DWORD, hFile AS DWORD, hFType AS DWORD, _
              txt AS STRING, txt2 AS STRING, zTxt AS ASCIIZ * 1000

        CompactPathToLabel CBHNDL, %IDC_PATHLBL, PBDirPath(CBHNDL, %IDC_DIR1, "")
        CONTROL SET FOCUS GetDlgItem(CBHNDL, %IDC_FILE1), %IDC_FILE1

     CASE %WM_COMMAND
        hDir   = GetDlgItem(GetDlgItem(CBHNDL, %IDC_DIR1), %IDC_DIR1)
        hDrive = GetDlgItem(GetDlgItem(CBHNDL, %IDC_DRIVE1), %IDC_DRIVE1)
        hFile  = GetDlgItem(GetDlgItem(CBHNDL, %IDC_FILE1), %IDC_FILE1)
        hFType = GetDlgItem(GetDlgItem(CBHNDL, %IDC_FTYPE1), %IDC_FTYPE1)

        SELECT CASE CBCTL
           CASE 3070 'Disable/Enable checkbox
              IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG =  1 THEN
                 CONTROL GET CHECK CBHNDL, CBCTL TO lRes
                 IF lRes THEN
                    CONTROL ENABLE CBHNDL, %IDC_FILETXT
                    EnableWindow hDrive, %TRUE : DIALOG DOEVENTS : DIALOG DOEVENTS : DIALOG DOEVENTS
                    EnableWindow hFType, %TRUE : DIALOG DOEVENTS
                    EnableWindow hFile, %TRUE  : DIALOG DOEVENTS
                    EnableWindow hDir, %TRUE   : DIALOG DOEVENTS
                 ELSE
                    CONTROL DISABLE CBHNDL, %IDC_FILETXT
                    EnableWindow GetDlgItem(CBHNDL, %IDC_FILETXT), 0
                    EnableWindow hDrive, 0 : DIALOG DOEVENTS : DIALOG DOEVENTS : DIALOG DOEVENTS
                    EnableWindow hFType, 0 : DIALOG DOEVENTS
                    EnableWindow hFile, 0  : DIALOG DOEVENTS
                    EnableWindow hDir, 0   : DIALOG DOEVENTS
                 END IF
              END IF

           CASE 3071 'Disable/Enable checkbox
              IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG =  1 THEN
                 CONTROL GET CHECK CBHNDL, CBCTL TO lRes
                 SendMessage hFtype, %DCM_SETFTYPESEL3DFACE, lRes, 0 'wParam toggles color
                 SendMessage hDrive, %DCM_SETDRVSEL3DFACE, lRes, 0
                 SendMessage hFile, %DCM_SETFILESEL3DFACE, lRes, 1   'lParam sets redraw
                 SendMessage hDir, %DCM_SETDIRSEL3DFACE, lRes, 1
              END IF

           CASE 3060 '&Ok
              IF SendMessage(hFile, %LB_GETCURSEL, 0, 0) <> %LB_ERR THEN
                 LISTBOX GET TEXT GetParent(hFile), %IDC_FILE1 TO txt
                 zTxt = PBDirPath(CBHNDL, %IDC_DIR1, "") + txt
                 MessageBox CBHNDL, zTxt, "Selected file:", %MB_OK OR %MB_ICONINFORMATION
                 SetFocus CBLPARAM
              ELSE
                 BEEP
              END IF

           CASE %IDCANCEL
              IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG =  1 THEN DIALOG END CBHNDL

           CASE %IDC_DRIVE1 'PBDRIVE COMBOBOX
              IF CBCTLMSG = %CBN_SELENDOK THEN
                 CompactPathToLabel CBHNDL, %IDC_PATHLBL, PBDirPath(CBHNDL, %IDC_DIR1, "")
              END IF

           CASE %IDC_DIR1   'PBDIR LISTBOX
              IF CBCTLMSG = %LBN_DBLCLK THEN
                 CompactPathToLabel CBHNDL, %IDC_PATHLBL, PBDirPath(CBHNDL, %IDC_DIR1, "")
              END IF

           CASE %IDC_FILETXT 'FILE NAME TEXTBOX
              IF CBCTLMSG = %EN_UPDATE THEN
                 CONTROL GET TEXT CBHNDL, %IDC_FILETXT TO txt
                 CONTROL SEND GetParent(hFile), %IDC_FILE1, %LB_GETCURSEL, 0, 0 TO cSel
                 CONTROL SEND GetParent(hFile), %IDC_FILE1, %LB_FINDSTRING, -1, STRPTR(txt) TO lRes
                 IF cSel = %LB_ERR  AND lRes <> %LB_ERR THEN 'if nothing selected and first part found
                    CONTROL SEND GetParent(hFile), %IDC_FILE1, %LB_SETTOPINDEX, lRes, 0
                 END IF
              END IF

           CASE %IDC_FILE1   'PBFILE LISTBOX
              IF CBCTLMSG = %LBN_SELCHANGE THEN
                 IF (GetWindowLong(hFile, %GWL_STYLE) AND %LBS_EXTENDEDSEL) OR _ 'if multiple sel style
                       (GetWindowLong(hFile, %GWL_STYLE) AND %LBS_MULTIPLESEL) THEN
                    CONTROL SEND GetParent(hFile), %IDC_FILE1, %LB_GETSELCOUNT, 0, 0 TO cSel
                    IF cSel > %LB_ERR THEN
                       REDIM selItems(cSel) AS LONG
                       CONTROL SEND GetParent(hFile), %IDC_FILE1, %LB_GETSELITEMS, cSel, VARPTR(selItems(0)) TO cSel
                       IF cSel <> %LB_ERR THEN
                          FOR ii = 0 TO cSel - 1
                             lRes = SendMessage(hFile, %LB_GETTEXTLEN, selItems(ii), 0)
                             txt2 = SPACE$(lRes)
                             lRes = SendMessage(hFile, %LB_GETTEXT, selItems(ii), BYVAL STRPTR(txt2))
                             txt = txt + txt2 + ", "
                          NEXT
                          txt = LEFT$(txt, LEN(txt) - 2) 'cut off final comma and space
                       END IF
                    END IF
                 ELSE
                    LISTBOX GET TEXT GetParent(hFile), %IDC_FILE1 TO txt
                 END IF
                 CONTROL SET TEXT CBHNDL, %IDC_FILETXT, txt

              ELSEIF CBCTLMSG = %LBN_DBLCLK THEN
                 LISTBOX GET TEXT GetParent(hFile), %IDC_FILE1 TO txt
                 zTxt = PBDirPath(CBHNDL, %IDC_DIR1, "") + txt
                 MessageBox CBHNDL, zTxt, "Selected file:", %MB_OK OR %MB_ICONINFORMATION
                 SetFocus hFile
              END IF

           CASE %IDC_FTYPE1   'PB FILE TYPE COMBOBOX
              IF CBCTLMSG = %CBN_SELENDOK THEN
                 COMBOBOX GET TEXT GetDlgItem(CBHNDL, %IDC_FTYPE1), %IDC_FTYPE1 TO txt
                 txt = TRIM$(txt, ")")                'trim away right paranthesis
                 txt = MID$(txt, INSTR(txt, "(") + 1) 'extract file type part
                 PBFilesToList hFile, PBDirPath(CBHNDL, %IDC_DIR1, ""), txt, _
                               %FILE_ATTRIBUTE_ARCHIVE OR %FILE_ATTRIBUTE_READONLY
              END IF

        END SELECT
  END SELECT

END FUNCTION

'
' Make given path fit into a label control via %DT_PATH_ELLIPSIS
' other ways exist, but this one works in all systems, from Win95 and up..
'
SUB CompactPathToLabel(BYVAL hDlg AS DWORD, BYVAL idLabel AS LONG, BYVAL path AS STRING)
  LOCAL hDC AS DWORD, hFont AS DWORD, hLabel AS DWORD, rc AS RECT

  hLabel = GetDlgItem(hDlg, idLabel)
  hFont  = SendMessage(hLabel, %WM_GETFONT, 0, 0)
  GetClientRect hLabel, rc

  hDC = GetDC(hLabel)
  hFont = SelectObject(hDC, hFont)
  DrawText hDC, BYVAL STRPTR(path), LEN(path), rc, _
           %DT_PATH_ELLIPSIS OR %DT_MODIFYSTRING
  SelectObject hDC, hFont
  ReleaseDc hLabel, hDc

  CONTROL SET TEXT hDlg, idLabel, path
END SUB
